@@ -17,6 +17,11 @@
 #include "rbbscommon.h"
 
 #define CONFFILE    "rbbs.conf"          /* name of the configuration file */
+#define TEXTTYPE    "text/plain"         /* content-type of plain text */
+#define HTMLTYPE    "text/html"          /* content-type of HTML */
+#define CSSTYPE     "text/css"           /* content-type of CSS */
+#define ATOMTYPE    "application/xml"    /* content-type of Atom */
+#define RXMLTYPE    "application/xml"    /* content-type of raw XML */
 #define ENCODING    "UTF-8"              /* encoding of the page */
 #define PETITBNUM   31                   /* bucket number of a petit map */
 #define RDATAMAX    1048576              /* max size of data to read */
@@ -59,6 +64,7 @@ const char *g_pagelang = NULL;           /* language of the page */
 const char *g_pagetitle = NULL;          /* title of the page */
 const char *g_pageauthor = NULL;         /* author of the page */
 const char *g_pagesummary = NULL;        /* summary of the page */
+int g_norobot = FALSE;                   /* whether to forbid robots */
 const char *g_homepageuri = NULL;        /* URI of the home page */
 const char *g_adminpass = NULL;          /* password for administration */
 const char *g_writepass = NULL;          /* password for writing */
@@ -195,6 +201,8 @@ int main(int argc, char **argv){
       g_pageauthor = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "pagesummary:")){
       g_pagesummary = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "norobot:")){
+      g_norobot = !strcmp(skiplabel(tmp), "true");
     } else if(cbstrfwimatch(tmp, "homepageuri:")){
       g_homepageuri = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "adminpass:")){
@@ -564,7 +572,7 @@ void xprintf(const char *format, ...){
 /* show the error page and exit */
 void senderror(const char *msg){
   printf("Status: 500 Internal Server Error\r\n");
-  printf("Content-Type: text/plain; charset=%s\r\n", ENCODING);
+  printf("Content-Type: %s; charset=%s\r\n", TEXTTYPE, ENCODING);
   printf("\r\n");
   printf("Error: %s\n", msg);
   exit(1);
@@ -574,7 +582,7 @@ void senderror(const char *msg){
 /* show the error page of not found */
 void sendnotfound(void){
   printf("Status: 404 Not Found\r\n");
-  printf("Content-Type: text/plain; charset=%s\r\n", ENCODING);
+  printf("Content-Type: %s; charset=%s\r\n", TEXTTYPE, ENCODING);
   printf("\r\n");
   printf("The target of the requested URI was not found\n");
 }
@@ -594,9 +602,8 @@ void senddump(void){
   tmp = dateforhttp(rbbsmdate(g_rbbs), FALSE);
   printf("Last-Modified: %s\r\n", tmp);
   free(tmp);
-  printf("Content-Type: application/xml\r\n");
+  printf("Content-Type: %s\r\n", RXMLTYPE);
   printf("\r\n");
-
   xprintf("<?xml version=\"1.0\" encoding=\"%@\"?>\n", ENCODING);
   xprintf("<!DOCTYPE rbbs SYSTEM \"rbbs.dtd\">\n");
   xprintf("<rbbs version=\"%@\" cdate=\"%@\" mdate=\"%@\">\n",
@@ -643,15 +650,15 @@ void sendfeed(void){
   tmp = dateforhttp(rbbsmdate(g_rbbs), FALSE);
   printf("Last-Modified: %s\r\n", tmp);
   free(tmp);
-  printf("Content-Type: application/xml\r\n");
+  printf("Content-Type: %s\r\n", ATOMTYPE);
   printf("\r\n");
   xprintf("<?xml version=\"1.0\" encoding=\"%@\"?>\n", ENCODING);
   xprintf("<feed version=\"0.3\" xmlns=\"http://purl.org/atom/ns#\" xml:lang=\"%@\">\n",
           g_pagelang);
   prefix = cbsprintf("%s?id=", g_systemuri);
   if(art){
-    xprintf("<link rel=\"alternate\" type=\"text/html\" href=\"%@?id=%@\"/>\n",
-            g_systemuri, artid(art));
+    xprintf("<link rel=\"alternate\" type=\"%@\" href=\"%@?id=%@\"/>\n",
+            HTMLTYPE, g_systemuri, artid(art));
     xprintf("<title>%@</title>\n", artsubject(art));
     xprintf("<author>\n");
     xprintf("<name>%@</name>\n", artauthor(art));
@@ -665,7 +672,7 @@ void sendfeed(void){
     free(xml);
     freearticle(art);
   } else {
-    xprintf("<link rel=\"alternate\" type=\"text/html\" href=\"%@\"/>\n", g_systemuri);
+    xprintf("<link rel=\"alternate\" type=\"%@\" href=\"%@\"/>\n", HTMLTYPE, g_systemuri);
     xprintf("<title>%@</title>\n", g_pagetitle);
     xprintf("<author>\n");
     xprintf("<name>%@</name>\n", g_pageauthor);
@@ -712,7 +719,7 @@ void sendhtml(void){
     printf("Set-Cookie: author=%s; expires=Thu, 1-Jan-2037 00:00:00 GMT\r\n", tmp);
     free(tmp);
   }
-  printf("Content-Type: text/html; charset=%s\r\n", ENCODING);
+  printf("Content-Type: %s; charset=%s\r\n", HTMLTYPE, ENCODING);
   printf("\r\n");
   xprintf("<?xml version=\"1.0\" encoding=\"%@\"?>\n", ENCODING);
   xprintf("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\""
@@ -721,14 +728,20 @@ void sendhtml(void){
           g_pagelang, g_pagelang);
   xprintf("<head>\n");
   xprintf("<title>%@</title>\n", g_pagetitle);
-  xprintf("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%@\" />\n",
-          ENCODING);
-  xprintf("<meta http-equiv=\"Content-Style-Type\" content=\"text/css\" />\n");
+  if(g_rbbs && (p_action == ACTVIEW || p_action == ACTLIST || p_action == ACTSEARCH)){
+    tmp = dateforhttp(rbbsmdate(g_rbbs), FALSE);
+    xprintf("<meta http-equiv=\"Last-Modified\" content=\"%@\" />\n", tmp);
+    free(tmp);
+  }
+  xprintf("<meta http-equiv=\"Content-Type\" content=\"%@; charset=%@\" />\n",
+          HTMLTYPE, ENCODING);
+  xprintf("<meta http-equiv=\"Content-Style-Type\" content=\"%@\" />\n", CSSTYPE);
   xprintf("<meta name=\"author\" content=\"%@\" />\n", g_pageauthor);
   xprintf("<meta name=\"generator\" content=\"RBBS %@\" />\n", RBBS_VERSION);
-  xprintf("<link rel=\"contents\" href=\"%@\" />\n", g_systemuri);
+  if(g_norobot) xprintf("<meta name=\"robots\" content=\"NOINDEX,NOFOLLOW\" />\n");
+  xprintf("<link rel=\"contents\" href=\"%@?action=%d\" />\n", g_systemuri, ACTLIST);
   if((tmp = cbreadfile(g_stylefile, NULL)) != NULL){
-    xprintf("<style type=\"text/css\">\n");
+    xprintf("<style type=\"%@\">\n", CSSTYPE);
     xprintf("%s", tmp);
     xprintf("</style>\n");
     free(tmp);
@@ -742,6 +755,7 @@ void sendhtml(void){
     showcaption();
   }
   showpagenavi();
+  xprintf("<hr />\n");
   switch(p_action){
   case ACTVIEW:
     showarticles();
@@ -850,6 +864,7 @@ void shownote(const char *msg){
 
 /* show the page navigation */
 void showpagenavi(void){
+  static int fst = TRUE;
   CBDATUM *buf;
   const char *fstr, *cond;
   buf = cbdatumopen("", 0);
@@ -869,34 +884,36 @@ void showpagenavi(void){
   if(g_homepageuri[0] != '\0')
     xprintf("<a href=\"%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
             g_homepageuri, ++g_tabidx, g_homelabel);
-  xprintf("<a href=\"%@?action=%d\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+  xprintf("<a href=\"%@?action=%d\" rel=\"help\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
           g_systemuri, ACTHELP, ++g_tabidx, g_helplabel);
-  xprintf("<a href=\"%@?action=%d\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+  xprintf("<a href=\"%@?action=%d\" class=\"navianc\" tabindex=\"%d\" accesskey=\"9\">%@</a>\n",
           g_systemuri, ACTARTFORM, ++g_tabidx, g_postlabel);
   fstr = FEEDCDATE;
   if(p_sort == SORTMDATEA || p_sort == SORTMDATED) fstr = FEEDMDATE;
-  xprintf("<a href=\"%@?feed=%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
-          g_systemuri, fstr, ++g_tabidx, g_feedlabel);
-  xprintf("<a href=\"%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
-          g_systemuri, ++g_tabidx, g_reloadlabel);
-  xprintf("<a href=\"%@?action=%d\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+  xprintf("<a href=\"%@?feed=%@\" type=\"%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+          g_systemuri, fstr, ATOMTYPE, ++g_tabidx, g_feedlabel);
+  xprintf("<a href=\"%@?action=%d\" rel=\"top\" class=\"navianc\""
+          " tabindex=\"%d\" accesskey=\"0\">%@</a>\n",
+          g_systemuri, ACTVIEW, ++g_tabidx, g_reloadlabel);
+  xprintf("<a href=\"%@?action=%d\" rel=\"search\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
           g_systemuri, ACTSEARCH, ++g_tabidx, g_searchlabel);
-  xprintf("<a href=\"%@?action=%d\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+  xprintf("<a href=\"%@?action=%d\" rel=\"contents\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
           g_systemuri, ACTLIST, ++g_tabidx, g_listlabel);
   if(p_page > 0){
-    xprintf("<a href=\"%@?page=%d%s\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+    xprintf("<a href=\"%@?page=%d%s\" rel=\"prev\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
             g_systemuri, p_page - 1, cond, ++g_tabidx, g_prevlabel);
   } else {
     xprintf("<span class=\"navivoid\">%@</span>\n", g_prevlabel);
   }
   if(g_cannext && p_action != ACTLIST){
-    xprintf("<a href=\"%@?page=%d%s\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+    xprintf("<a href=\"%@?page=%d%s\" rel=\"next\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
             g_systemuri, p_page + 1, cond, ++g_tabidx, g_nextlabel);
   } else {
     xprintf("<span class=\"navivoid\">%@</span>\n", g_nextlabel);
   }
   xprintf("</div>\n");
   cbdatumclose(buf);
+  fst = FALSE;
 }
 
 
@@ -906,6 +923,7 @@ void showcaption(void){
   xprintf("<h1>%@</h1>\n", g_pagetitle);
   xprintf("<p>%@</p>\n", g_pagesummary);
   xprintf("</div>\n");
+  xprintf("<hr />\n");
 }
 
 
@@ -971,6 +989,7 @@ void showlist(void){
   }
   if(snum < 1) xprintf("<span class=\"note\">There is no corresponding article.</span>\n");
   xprintf("</div>\n");
+  xprintf("<hr />\n");
   if(snum > g_shownum) g_cannext = TRUE;
 }
 
@@ -1075,6 +1094,7 @@ void showsearchform(void){
   xprintf("<input type=\"hidden\" name=\"action\" value=\"%d\" />\n", ACTSEARCH);
   xprintf("</div>\n");
   xprintf("</form>\n");
+  xprintf("<hr />\n");
 }
 
 
@@ -1116,15 +1136,21 @@ void showarticles(void){
 
 /* show one article with the navigation control */
 void showartwithctl(ARTICLE *art){
-  char *html;
+  static int anum = 0;
+  char akey[NUMBUFSIZ], *html;
+  if(++anum < 9){
+    sprintf(akey, "%d", anum);
+  } else {
+    sprintf(akey, "#");
+  }
   xprintf("<div class=\"artctl\">\n");
   xprintf("<div class=\"artnavi\">\n");
   xprintf("<a href=\"%@?action=%d&amp;id=%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
           g_systemuri, ACTARTFORM, artid(art), ++g_tabidx, g_editlabel);
-  xprintf("<a href=\"%@?feed=%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
-          g_systemuri, artid(art), ++g_tabidx, g_feedlabel);
-  xprintf("<a href=\"%@?id=%@&amp;focus=%d\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
-          g_systemuri, artid(art), FCSDETAIL, ++g_tabidx, g_focuslabel);
+  xprintf("<a href=\"%@?feed=%@\" type=\"%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+          g_systemuri, artid(art), ATOMTYPE, ++g_tabidx, g_feedlabel);
+  xprintf("<a href=\"%@?id=%@&amp;focus=%d\" class=\"navianc\" tabindex=\"%d\" accesskey=\"%s\">"
+          "%@</a>\n", g_systemuri, artid(art), FCSDETAIL, ++g_tabidx, akey, g_focuslabel);
   xprintf("</div>\n");
   html = arttohtml(art, p_focus != FCSSIMPLE ? 0 : g_blknum, p_focus == FCSDETAIL ? 0 : g_resnum);
   xprintf("%s", html);
@@ -1154,6 +1180,7 @@ void showartwithctl(ARTICLE *art){
     xprintf("</form>\n");
   }
   xprintf("</div>\n");
+  xprintf("<hr />\n");
 }
 
 
@@ -1264,6 +1291,7 @@ void showartform(void){
   xprintf("<input type=\"hidden\" name=\"password\" value=\"%@\" />\n", p_password);
   xprintf("</div>\n");
   xprintf("</form>\n");
+  xprintf("<hr />\n");
 }
 
 
@@ -1272,9 +1300,12 @@ void showartconfirm(void){
   ARTICLE *art;
   CBDATUM *buf;
   char *xml, *tmp;
+  int len;
   buf = cbdatumopen("", 0);
   if(p_id[0] != '\0'){
-    xsprintf(buf, "%s", p_text);
+    xml = strnormalize(p_text, FALSE, TRUE);
+    xsprintf(buf, "%s", xml);
+    free(xml);
   } else {
     xsprintf(buf, "<article>\n");
     xsprintf(buf, "<head>\n");
@@ -1299,9 +1330,10 @@ void showartconfirm(void){
     xsprintf(buf, "</tail>\n");
     xsprintf(buf, "</article>\n");
   }
+  len = cbdatumsize(buf);
   art = makearticle(cbdatumptr(buf));
   cbdatumclose(buf);
-  if(p_id[0] != '\0' && p_text[0] == '\0'){
+  if(p_id[0] != '\0' && len < 1){
     shownote("The article is being deleted.");
     xprintf("<form method=\"post\" action=\"%@\">\n", g_systemuri);
     xprintf("<div class=\"comform\">\n");
@@ -1369,6 +1401,7 @@ void showartconfirm(void){
     xprintf("<input type=\"hidden\" name=\"action\" value=\"%d\" />\n", ACTPOSTART);
     xprintf("<input type=\"hidden\" name=\"id\" value=\"%@\" />\n", p_id);
     xprintf("<input type=\"hidden\" name=\"password\" value=\"%@\" />\n", p_password);
+    xprintf("<input type=\"hidden\" name=\"author\" value=\"%@\" />\n", p_author);
     tmp = cbbaseencode(xml, -1);
     xprintf("<input type=\"hidden\" name=\"text\" value=\"%@\" />\n", tmp);
     free(tmp);
@@ -1378,6 +1411,7 @@ void showartconfirm(void){
   }
   freearticle(art);
   showartformback();
+  xprintf("<hr />\n");
 }
 
 
@@ -1473,6 +1507,7 @@ void mypostarticle(void){
   }
   freearticle(art);
   free(xml);
+  xprintf("<hr />\n");
 }
 
 
@@ -1567,6 +1602,7 @@ void showhelp(void){
   xprintf("</div>\n");
   freearticle(art);
   free(xml);
+  xprintf("<hr />\n");
 }
 
 
