@@ -887,6 +887,7 @@ char *arttotext(ARTICLE *art){
   xsprintf(buf, "%s\n", art->author);
   xsprintf(buf, "%s\n", art->cdate);
   xsprintf(buf, "%s\n", art->mdate);
+  xsprintf(buf, "\n");
   for(i = 0; i < cblistnum(art->body); i++){
     blk = cblistval(art->body, i, NULL);
     elems = cbxmlbreak(blk, FALSE);
@@ -903,6 +904,7 @@ char *arttotext(ARTICLE *art){
     xsprintf(buf, "\n");
     cblistclose(elems);
   }
+  xsprintf(buf, "\n");
   for(i = 0; i < cblistnum(art->tail); i++){
     blk = cblistval(art->tail, i, NULL);
     elems = cbxmlbreak(blk, FALSE);
@@ -1082,27 +1084,25 @@ char *dateformen(const char *date){
    `trim' specifies whether to cut the heading spaces and tailing spaces.
    The return value is a converted string. */
 char *strnormalize(const char *str, int nolf, int trim){
-  char *buf, *wp;
+  char *buf, *rp, *wp;
+  int miss;
   assert(str);
-  buf = cbmemdup(str, -1);
+  if((buf = cbiconv(str, -1, "UTF-8", "UTF-8", NULL, &miss)) != NULL && miss > 0){
+    free(buf);
+    buf = cbiconv(str, -1, "UTF-8", "US-ASCII", NULL, NULL);
+  }
+  if(!buf) buf = cbmemdup(str, -1);
+  rp = buf;
   wp = buf;
-  while(*str != '\0'){
-    switch(*str){
-    case '\r':
-      break;
-    case '\n':
+  while(*rp != '\0'){
+    if(*rp == '\n'){
       *(wp++) = nolf ? ' ' : '\n';
-      break;
-    case '\t':
-    case '\v':
-    case '\f':
-    case 0x1b:
+    } else if(*rp == '\t' || *rp == '\v' || *rp == '\f'){
       *(wp++) = ' ';
-      break;
-    default:
-      *(wp++) = *str;
+    } else if(*rp < 0 || (*rp >= ' ' && *rp <= '~')){
+      *(wp++) = *rp;
     }
-    str++;
+    rp++;
   }
   *wp = '\0';
   if(trim) cbstrtrim(buf);
