@@ -52,7 +52,7 @@
 #ifndef RBBS_VERSION                     /* version and URI */
 #define RBBS_VERSION   "0.0.0"
 #endif
-#define RBBS_URI       "http://rbbs.sourceforge.net/"
+#define RBBS_URI       "http://rbbs.sourceforge.jp/"
 
 #undef TRUE                              /* boolean true */
 #define TRUE           1
