@@ -16,6 +16,7 @@
 
 #include "rbbscommon.h"
 
+#define HTMLTYPE    "text/html"          /* content-type of HTML */
 #define ENCODING    "UTF-8"              /* encoding of the page */
 
 enum {                                   /* enumeration for the format of output data */
@@ -463,6 +464,8 @@ int procdump(const char *name, int ftype){
       printf("%s\n", g_dumplang);
       printf("%s\n", g_dumptitle);
       printf("%s\n", g_dumpauthor);
+      printf("%s\n", rbbscdate(rbbs));
+      printf("%s\n", rbbsmdate(rbbs));
       printf("%s\n", g_dumpsummary);
       break;
     case FTHTML:
@@ -473,8 +476,11 @@ int procdump(const char *name, int ftype){
               g_dumplang, g_dumplang);
       xprintf("<head>\n");
       xprintf("<title>%@</title>\n", g_dumptitle);
-      xprintf("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%@\" />\n",
-              ENCODING);
+      tmp = dateforhttp(rbbsmdate(rbbs), FALSE);
+      xprintf("<meta http-equiv=\"Last-Modified\" content=\"%@\" />\n", tmp);
+      free(tmp);
+      xprintf("<meta http-equiv=\"Content-Type\" content=\"%@; charset=%@\" />\n",
+              HTMLTYPE, ENCODING);
       xprintf("<meta name=\"author\" content=\"%@\" />\n", g_dumpauthor);
       xprintf("<meta name=\"generator\" content=\"RBBS %@\" />\n", RBBS_VERSION);
       xprintf("<link rel=\"contents\" href=\"%@\" />\n", g_rbbsuri);
@@ -488,7 +494,7 @@ int procdump(const char *name, int ftype){
       xprintf("<?xml version=\"1.0\" encoding=\"%@\"?>\n", ENCODING);
       xprintf("<feed version=\"0.3\" xmlns=\"http://purl.org/atom/ns#\" xml:lang=\"%@\">\n",
               g_dumplang);
-      xprintf("<link rel=\"alternate\" type=\"text/html\" href=\"%@\"/>\n", g_rbbsuri);
+      xprintf("<link rel=\"alternate\" type=\"%@\" href=\"%@\"/>\n", HTMLTYPE, g_rbbsuri);
       xprintf("<title>%@</title>\n", g_dumptitle);
       xprintf("<author>\n");
       xprintf("<name>%@</name>\n", g_dumpauthor);
